@@ -1,48 +1,105 @@
 colsOut <- c("lhs", "op", "rhs", "est", "std.error",
               "z.value", "p.value", "ci.lower", "ci.upper")
 header <- c("Variable", "op", "Variable", "Estimate",
-            "Std.Error", "z.value", "P(>|z|)", "CI.Lower", "CI.Upper")
+            "Std.Error", "z.value", "P(>|z|)", "ci.lower", "ci.upper")
 
 
-formatParTable <- function(parTable, digits = 3, scientific = FALSE,
-                           ci = FALSE, width = 14) {
-  parTable <- fillColsParTable(parTable)
-
-  isStructOrMeasure <- parTable$op %in% c("~", "=~", "~~") &
-    parTable$lhs != parTable$rhs
-  parTable$lhs[isStructOrMeasure] <-
-    paste(parTable$lhs[isStructOrMeasure], parTable$op[isStructOrMeasure])
-  isLabel  <- parTable$op == ":="
-  parTable[isLabel, "label"] <- ""
+shorten <- \(x, n) abbreviate(x, minlength = n, strict = TRUE)
 
 
-  isResVar <- parTable$op == "~~" & parTable$lhs == parTable$rhs
-  parTable$lhs[parTable$op == "~1" | isResVar | isLabel] <-
-    pasteLabels(parTable$lhs[parTable$op == "~1" | isResVar | isLabel],
-                parTable$label[parTable$op == "~1" | isResVar | isLabel],
-                width = width)
-  parTable$rhs[parTable$op != "~1"] <-
-    pasteLabels(parTable$rhs[parTable$op != "~1"],
-                parTable$label[parTable$op != "~1"], width = width)
+formatParTable <- function(parTable, 
+                           digits = 3, 
+                           scientific = FALSE,
+                           ci = FALSE, 
+                           width.min = 14, 
+                           pad.res = TRUE, 
+                           extra.cols = NULL, 
+                           shorten.lhs.header = FALSE) {
+  parTable    <- fillColsParTable(parTable)
+  parTable.in <- parTable # unformatted partable
+
+  isCustom <- parTable$op == ":="
+  isVar    <- parTable$op == "~~" & parTable$lhs == parTable$rhs
+  isCov    <- parTable$op == "~~" & parTable$lhs != parTable$rhs
+  isReg    <- parTable$op == "~"
+  isIntr   <- parTable$op == "~1" # Intercept
+  isMeasr  <- parTable$op == "=~"
+
+  parTable[isCustom, "label"] <- ""
+
+  isDoubleCol <- isReg  | isCov | isMeasr # Should output have double columns?
+  isSingleCol <- isIntr | isVar | isCustom
+
+
+  if (shorten.lhs.header) {
+    lhsDouble <- paste(shorten(parTable$lhs[isDoubleCol], n = width - 2L),
+                       parTable$op[isDoubleCol])
+  } else {
+    lhsDouble <- paste(parTable$lhs[isDoubleCol], parTable$op[isDoubleCol])
+  }
+
+  width <- max(width.min, maxchar(lhsDouble))
+
+  lhsSingle <- pasteLabels(parTable$lhs[isSingleCol],
+                           parTable$label[isSingleCol],
+                           width = width - pad.res)
+  rhsDouble <- pasteLabels(parTable$rhs[isDoubleCol],
+                           parTable$label[isDoubleCol],
+                           width = width - pad.res)
 
-  parTable$lhs[!isStructOrMeasure] <-
-    format(parTable$lhs[!isStructOrMeasure], width = width, justify = "left")
-  parTable$rhs <- format(parTable$rhs, width = width, justify = "left")
   parTable$p.value <- formatPval(parTable$p.value, scientific = scientific)
 
-  if (!ci) {
-    header <- header[!grepl("CI", header)]
-    colsOut <- colsOut[!grepl("ci", colsOut)]
-  }
-  parTable <- parTable[colsOut]
+  
+  ftext <- \(text, n = width - pad.res) format(text, width = n, justify = "left")
 
-  for (i in seq_len(length(colsOut) - 3) + 3) { # skip first 3 (lhs, op, rhs)
-    if (is.numeric(parTable[[i]])) parTable[[i]] <- round(parTable[[i]], digits)
-    maxWidth <- maxchar(c(header[[i]], parTable[[i]]))
-    parTable[[i]] <- format(parTable[[i]], width = maxWidth,
-                            digits = digits, justify = "right", scientific=FALSE)
-    parTable[[i]] <- stringr::str_replace_all(parTable[[i]], "NA", "  ")
-    header[[i]] <- format(header[[i]], width = maxWidth, justify = "right")
+  parTable[isDoubleCol, "lhs"] <- ftext(lhsDouble)
+  parTable[isDoubleCol, "rhs"] <- ftext(rhsDouble)
+  parTable[isSingleCol, "lhs"] <- ftext(lhsSingle)
+  parTable[isSingleCol, "rhs"] <- ""
+  
+  if (pad.res) {
+    etas    <- getEtas(parTable.in, isLV = FALSE)
+    inds    <- getInds(parTable.in)
+    resvars <- c(etas, inds)
+
+    isResVarCovOrInt <- (
+      (isVar | isCov | isIntr) & 
+      (parTable.in$rhs %in% resvars | parTable.in$lhs %in% resvars)
+    )
+
+    padVarCov <- ifelse(isResVarCovOrInt, yes = ".", no = " ")
+
+    parTable$lhs <- paste0(padVarCov, parTable$lhs) 
+    parTable$rhs <- paste0(padVarCov, parTable$rhs) 
+  }
+
+  if (!ci) {
+    ci.pattern <- "^ci\\.(lower|upper)$"
+    header  <- header[!grepl(ci.pattern, header)]
+    colsOut <- colsOut[!grepl(ci.pattern, colsOut)]
+  }
+
+  if (!is.null(extra.cols)) {
+    header  <- c(header, extra.cols)
+    colsOut <- c(colsOut, extra.cols)
+  }
+
+  parTable      <- parTable[colsOut]
+  names(header) <- colsOut
+
+  valueCols <- setdiff(colnames(parTable), c("lhs", "op", "rhs"))
+
+  for (col in valueCols) { # skip lhs, op and rhs
+    if (is.numeric(parTable[[col]])) 
+      parTable[[col]] <- round(parTable[[col]], digits)
+
+    maxWidth        <- maxchar(c(header[[col]], parTable[[col]]))
+    parTable[[col]] <- format(parTable[[col]], width = maxWidth,
+                              digits = digits, justify = "right", 
+                              scientific=FALSE)
+
+    parTable[[col]] <- stringr::str_replace_all(parTable[[col]], "NA", "  ")
+    header[[col]]   <- format(header[[col]], width = maxWidth, justify = "right")
   }
 
   list(parTable = parTable, header = header)
@@ -51,77 +108,109 @@ formatParTable <- function(parTable, digits = 3, scientific = FALSE,
 
 printParTable <- function(parTable,
                           scientific = FALSE,
-                          ci = FALSE, digits = 3,
+                          ci = FALSE, 
+                          digits = 3,
                           loadings = TRUE,
                           regressions = TRUE,
                           covariances = TRUE,
                           intercepts = TRUE,
                           variances = TRUE,
                           custom = TRUE,
-                          padWidth = 2,
+                          extra.cols = NULL,
+                          padWidth = 1, # we already pad lhs/rhs with "."/" "
                           padWidthLhs = 2,
                           spacing = 2) {
   formatted <- formatParTable(parTable, digits = digits,
-                              ci = ci, scientific = scientific)
+                              ci = ci, scientific = scientific,
+                              extra.cols = extra.cols)
   fParTable <- formatted$parTable
-  header <- formatted$header
-  lhs <- unique(fParTable$lhs)
+  header    <- formatted$header
+  lhs       <- unique(fParTable$lhs)
 
-  pad <- stringr::str_dup(" ", padWidth + padWidthLhs +
-                          maxchar(fParTable$rhs) - 1)
-  space <- stringr::str_dup(" ", spacing)
+  fullPadWidth <- padWidth + padWidthLhs + maxchar(fParTable$rhs) - 1
+  pad          <- stringr::str_dup(" ", fullPadWidth)
+  space        <- stringr::str_dup(" ", spacing)
 
-  formattedHeader <-
-    paste0(pad, stringr::str_c(header[-(1:3)], collapse = space), "\n")
+  formattedHeader <- paste0(
+    pad, stringr::str_c(header[-(1:3)], collapse = space), "\n"
+  )
 
   # Measurement model
   parTableLoadings <- fParTable[fParTable$op == "=~", ]
   if (loadings && NROW(parTableLoadings) > 0) {
     cat("Latent Variables:\n", formattedHeader)
-    printParTableDouble(parTableLoadings, padWidth = padWidth, padWidthLhs = padWidthLhs,
-                        spacing = spacing)
+
+    printParTableDouble(
+      parTableLoadings, 
+      padWidth = padWidth, 
+      padWidthLhs = padWidthLhs,
+      spacing = spacing
+    )
   }
 
   # Regressions
   parTableRegressions <- fParTable[parTable$op == "~", ]
   if (regressions && NROW(parTableRegressions) > 0) {
     cat("\nRegressions:\n", formattedHeader)
-    printParTableDouble(parTableRegressions, padWidth = padWidth, padWidthLhs = padWidthLhs,
-                        spacing = spacing)
+
+    printParTableDouble(
+      parTableRegressions, 
+      padWidth = padWidth, 
+      padWidthLhs = padWidthLhs,
+      spacing = spacing
+    )
   }
 
   # Intercepts
   parTableIntercepts <- fParTable[parTable$op == "~1", ]
   if (intercepts && NROW(parTableIntercepts) > 0) {
     cat("\nIntercepts:\n", formattedHeader)
-    printParTableSingle(parTableIntercepts, padWidth = padWidth, padWidthLhs = padWidthLhs,
-                        spacing = spacing)
+
+    printParTableSingle(
+      parTableIntercepts,
+      padWidth = padWidth, 
+      padWidthLhs = padWidthLhs,
+      spacing = spacing
+    )
   }
 
   # Covariances
   parTableCovariances <- fParTable[parTable$op == "~~" & parTable$lhs != parTable$rhs, ]
   if (covariances && NROW(parTableCovariances) > 0) {
     cat("\nCovariances:\n", formattedHeader)
-    printParTableDouble(parTableCovariances, padWidth = padWidth, padWidthLhs = padWidthLhs,
-                        spacing = spacing)
+
+    printParTableDouble(
+      parTableCovariances,
+      padWidth = padWidth,
+      padWidthLhs = padWidthLhs,
+      spacing = spacing
+    )
   }
 
   # Variances
   parTableVariances <- fParTable[parTable$op == "~~" & parTable$lhs == parTable$rhs, ]
   if (variances && NROW(parTableVariances) > 0) {
     cat("\nVariances:\n", formattedHeader)
-    printParTableSingle(parTableVariances, padWidth = padWidth, padWidthLhs = padWidthLhs,
-                        spacing = spacing)
 
+    printParTableSingle(
+      parTableVariances,
+      padWidth = padWidth,
+      padWidthLhs = padWidthLhs,
+      spacing = spacing
+    )
   }
 
   # Defined parameters
   parTableCustom <- fParTable[parTable$op == ":=", ]
   if (custom && NROW(parTableCustom) > 0) {
     cat("\nDefined Parameters:\n", formattedHeader)
-    printParTableSingle(parTableCustom, padWidth = padWidth, padWidthLhs = padWidthLhs,
-                        spacing = spacing)
 
+    printParTableSingle(
+      parTableCustom,
+      padWidth = padWidth,
+      padWidthLhs = padWidthLhs,
+      spacing = spacing
+    )
   }
   cat("\n")
 }
@@ -168,13 +257,20 @@ printRowsParTable <- function(lhs, rhs, padWidthLhs = 2,
 }
 
 
-pasteLabels <- function(vars, labels, width = 14, widthVar = 7, widthLabel = 4) {
+pasteLabels <- function(vars, labels, width = 14) {
+  initWidthVar   <- round(width / 2)
+  widthLabel     <- min(maxchar(labels), width - initWidthVar - 3L) # space + ()
+  widthVar       <- width - widthLabel - 3L
+
+  warnif(widthVar + widthLabel + 3L != width, "Mismatching widhts!")
+
   pasted <- paste0(vars, " (", labels, ")")
   widths <- nchar(pasted)
-  vars[widths > width] <-
-    abbreviate(vars[widths > width], minlength = widthVar)
-  labels[widths > width] <-
-    abbreviate(labels[widths > width], minlength = widthLabel)
+
+  wide         <- widths > width 
+  vars[wide]   <- shorten(vars[wide], n = widthVar)
+  labels[wide] <- shorten(labels[wide], n = widthLabel)
+
   labels[labels != ""] <- paste0("(", labels[labels != ""], ")")
 
   for (i in seq_along(vars)) {
@@ -223,9 +319,16 @@ getWidthPrintedParTable <- function(parTable,
                                     variances   = TRUE,
                                     padWidth    = 2,
                                     padWidthLhs = 2,
-                                    spacing     = 2) {
-  formatted <- formatParTable(parTable, digits = digits,
-                              ci = ci, scientific = scientific)
+                                    spacing     = 2,
+                                    extra.cols = NULL) {
+  formatted <- formatParTable(
+    parTable, 
+    digits = digits,
+    ci = ci, 
+    scientific = scientific,
+    extra.cols = extra.cols,
+  )
+
   fParTable <- formatted$parTable
   header    <- formatted$header
   lhs       <- unique(fParTable$lhs)
