@@ -12,6 +12,7 @@ R_DCROSS = "\u2561" # right double cross
 L_DCROSS = "\u255e" # left double cross
 U_CROSS = "\u2534" # up cross
 
+
 #' Get the simple slopes of a SEM model
 #'
 #' This function calculates simple slopes (predicted values of the outcome variable)
@@ -62,25 +63,29 @@ U_CROSS = "\u2534" # up cross
 #'
 #' @details
 #' \strong{Computation Steps}  
-#' 1. The function extracts parameter estimates (and, if necessary, their covariance
-#'    matrix) from the fitted SEM model (\code{model}).  
-#' 2. It identifies the coefficients for \code{x}, \code{z}, and \code{x:z} in the model's
-#'    parameter table, as well as the variance of \code{x}, \code{z}, and the residual.  
-#' 3. If \code{xz} is not provided, it will be constructed by combining \code{x} and
-#'    \code{z} with a colon (\code{":"}). In certain SEM software, the colon may be
-#'    removed or replaced internally; the function attempts to reconcile that.  
-#' 4. A grid of \code{x} and \code{z} values is created from \code{vals_x} and
-#'    \code{vals_z}. If \code{rescale = TRUE}, these values are transformed back into raw
-#'    metric units for display in the output.  
-#' 5. For each point in the grid, a predicted value of \code{y} is computed via
-#'    \code{(beta0 + beta_x * x + beta_z * z + beta_xz * x * z)} and, if included, a
-#'    mean offset.  
-#' 6. The standard error (\code{std.error}) is derived from the covariance matrix of
-#'    the relevant parameters, and if \code{ci_type = "prediction"}, adds the residual
-#'    variance.  
-#' 7. Confidence (or prediction) intervals are formed using \code{ci_width} (defaulting
-#'    to 95\%). The result is a table-like data frame with predicted values, CIs,
-#'    standard errors, z-values, and p-values.  
+#' \enumerate{
+#'    \item The function extracts parameter estimates (and, if necessary, their covariance
+#'          matrix) from the fitted SEM model (\code{model}).  
+#'    \item It identifies the coefficients for \code{x}, \code{z}, and \code{x:z} in the model's
+#'          parameter table, as well as the variance of \code{x}, \code{z} and the residual variance
+#'          of \code{y}.
+#'    \item If \code{xz} is not provided, it will be constructed by combining \code{x} and
+#'          \code{z} with a colon (\code{":"}). Depending on the approach used to estimate the model, 
+#'          the colon may be removed or replaced internally; the function attempts to reconcile that.  
+#'
+#'   \item A grid of \code{x} and \code{z} values is created from \code{vals_x} and
+#'         \code{vals_z}. If \code{rescale = TRUE}, these values are transformed back into raw
+#'         metric units for display in the output.  
+#'   \item For each point in the grid, a predicted value of \code{y} is computed via
+#'         \code{(beta0 + beta_x * x + beta_z * z + beta_xz * x * z)} and, if included, a
+#'         mean offset.  
+#'   \item The standard error (\code{std.error}) is derived from the covariance matrix of
+#'         the relevant parameters, and if \code{ci_type = "prediction"}, adds the residual
+#'         variance.  
+#'   \item Confidence (or prediction) intervals are formed using \code{ci_width} (defaulting
+#'         to 95\%). The result is a table-like data frame with predicted values, CIs,
+#'         standard errors, z-values, and p-values.  
+#' }
 #'
 #' @return A \code{data.frame} (invisibly inheriting class \code{"simple_slopes"})
 #' with columns:
@@ -146,7 +151,7 @@ simple_slopes <- function(x,
     xz <- stringr::str_remove_all(xz, ":")
 
   if (standardized) {
-    parTable <- standardized_estimates(model)
+    parTable <- standardized_estimates(model, correction = TRUE)
   } else parTable <- parameter_estimates(model)
 
   parTable <- getMissingLabels(parTable)
