@@ -18,26 +18,14 @@ warnif <- function(cond, ...) {
 }
 
 
-# utils for all methods
 calcCovParTable <- function(x, y, parTable, measurement.model = FALSE, maxlen = 100) {
-  if (measurement.model) {
-    parTable <- redefineMeasurementModel(parTable)
-  }
-  
-  parTable <- prepParTable(parTable[c("lhs", "op", "rhs", "est")], paramCol = "est")
-
   stopif(length(x) != length(y), "x and y must be the same length")
 
-  covs <- structure(numeric(length(x)), names = paste0(x, "~~", y))
+  if (measurement.model) parTable <- redefineMeasurementModel(parTable)
+  parTable <- parTable[!parTable$op %in% c(":=", "~1", "=~"), ]
+  parTable <- prepParTable(parTable[c("lhs", "op", "rhs", "est")], paramCol = "est")
 
-  for (i in seq_along(x)) {
-    paths <- tracePathsRecursively(x = x[i], y = y[i], pt = parTable, 
-                                   paramCol = "est", maxlen = maxlen)
-
-    covs[i] <- sum(vapply(paths, FUN.VALUE = numeric(1L), FUN = prod))
-  }
-
-  covs
+  tracePathsNumericCpp(x = x, y = y, parTable = parTable)
 }
 
 
@@ -159,7 +147,19 @@ getHigherOrderLVs <- function(parTable) {
 }
 
 
+isHigherOrderParTable <- function(parTable) {
+  length(getHigherOrderLVs(parTable)) > 0
+}
+
+
+isClustered <- function(object) {
+  length(modsem_inspect(object, what = "cluster")) > 0
+}
+
+
 getIndsLVs <- function(parTable, lVs) {
+  if (!length(lVs)) return(NULL)
+
   measrExprs <- parTable[parTable$op == "=~" & parTable$lhs %in% lVs, ]
   stopif(!NROW(measrExprs), "No measurement expressions found, for", lVs)
   lapplyNamed(lVs, FUN = function(lV) measrExprs[measrExprs$lhs == lV, "rhs"],
@@ -201,7 +201,7 @@ fillColsParTable <- function(parTable) {
   colNames <- c("lhs", "op", "rhs", "label", "est",
                 "std.error", "z.value", "p.value", "ci.lower", "ci.upper")
   parTable[colNames[!colNames %in% colnames(parTable)]] <- NA
-  parTable[colNames]
+  parTable #[colNames]
 }
 
 
@@ -210,7 +210,7 @@ getUniqueCombos <- function(x, match = FALSE) {
   # Base case, x is 1 length long and there are no unique combos
   if (length(x) <= 1) return(NULL)
 
-  rest <- getUniqueCombos(x[-1], match = FALSE)
+  rest <- getUniqueCombos(x[-1], match = match)
   combos <- data.frame(V1 = rep(x[[1]], length(x) - 1),
                        V2 = x[-1])
   if (match) combos <- rbind(data.frame(V1 = x, V2 = x), combos)
@@ -250,10 +250,10 @@ isModsemObject <- function(x) {
 }
 
 
-getIntercept <- function(x, parTable) {
+getIntercept <- function(x, parTable, col = "est") {
   if (length(x) > 1) stop2("x must be a single string")
 
-  intercept <- parTable[parTable$lhs == x & parTable$op == "~1", "est"]
+  intercept <- parTable[parTable$lhs == x & parTable$op == "~1", col]
 
   if (length(intercept) == 0) return(0)
   intercept
@@ -275,7 +275,7 @@ getMean <- function(x, parTable) {
   gamma <- parTable[parTable$lhs == x & parTable$op == "~", , drop = FALSE]
 
   if (NROW(gamma) == 0) return(meanY)
-  for (i in NROW(gamma)) {
+  for (i in seq_len(NROW(gamma))) {
     meanX <- getMean(gamma[i, "rhs"], parTable = parTable)
     meanY <- meanY + gamma[i, "est"] * meanX
   }
@@ -292,21 +292,23 @@ getMeans <- function(x, parTable) {
 }
 
 
-centerInteraction <- function(parTable) {
+centerInteractions <- function(parTable, center.means = TRUE) {
+  parTable <- meanInteractions(parTable)
   rows <- getIntTermRows(parTable)
-  for (i in NROW(rows)) {
+
+  for (i in seq_len(NROW(rows))) {
     Y <- rows[i, "lhs"]
     XZ <- unlist(stringr::str_split(rows[i, "rhs"], ":"))
     X <- XZ[[1]]
     Z <- XZ[[2]]
-
+    
     meanX <- getMean(X, parTable)
     meanZ <- getMean(Z, parTable)
 
     gammaXZ <- rows[i, "est"]
     gamma <- parTable[parTable$lhs == Y & parTable$op == "~", , drop = FALSE]
-    gammaX <- gamma[gamma$rhs == X, "est"] + gammaXZ * meanZ
 
+    gammaX <- gamma[gamma$rhs == X, "est"] + gammaXZ * meanZ
     gammaZ <- gamma[gamma$rhs == Z, "est"] + gammaXZ * meanX
 
     parTable[parTable$lhs == Y & parTable$op == "~" &
@@ -314,6 +316,49 @@ centerInteraction <- function(parTable) {
 
     parTable[parTable$lhs == Y & parTable$op == "~" &
              parTable$rhs == Z, "est"] <- gammaZ
+
+  }
+
+  if (center.means) {
+    innerVars <- unique(unlist(parTable[parTable$op == "~", c("rhs", "lhs")]))
+    parTable[parTable$lhs %in% innerVars & parTable$op == "~1", "est"] <- 0
+  }
+
+  parTable
+}
+
+
+meanInteractions <- function(parTable, ignore.means = FALSE) {
+  intTerms <- unique(parTable[grepl(":", parTable$rhs), "rhs"])
+
+  # remove existing
+  parTable <- parTable[!(parTable$op == "~1" & parTable$lhs %in% intTerms), 
+                       , drop = FALSE]
+ 
+  present  <- colnames(parTable)
+  newcols  <- c("lhs", "op", "rhs", "label", "est")
+  newcols  <- intersect(newcols, present)
+  fillcols <- setdiff(present, newcols) 
+
+  for (intTerm in intTerms) {
+    XZ <- unlist(stringr::str_split(intTerm, ":"))
+    X <- XZ[[1]]
+    Z <- XZ[[2]]
+ 
+    meanX <- if (!ignore.means) getMean(X, parTable) else 0
+    meanZ <- if (!ignore.means) getMean(Z, parTable) else 0
+
+    covXZ <- calcCovParTable(x = X, y = Z, parTable = parTable)
+    meanXZ <- meanX * meanZ + covXZ
+
+    newRow <- data.frame(lhs = intTerm, op = "~1", rhs = "",
+                         label = "", est = meanXZ)
+
+    # Get correct cols
+    newRow <- newRow[newcols]
+    newRow[fillcols] <- NA
+
+    parTable <- rbind(parTable, newRow)
   }
 
   parTable
@@ -342,15 +387,15 @@ isRowInParTable <- function(row, pt, ignore = NULL) {
 }
 
 
-rename <- function(X, ...) {
+rename <- function(.X, ...) {
   newNames <- list(...)
   oldNames <- names(newNames)
 
   for (old in oldNames) {
-    names(X)[names(X) == old] <- newNames[[old]]
+    names(.X)[names(.X) == old] <- newNames[[old]]
   }
 
-  X
+  .X
 }
 
 
@@ -368,7 +413,10 @@ clearConsoleLine <- function() {
 getDiffTwoMax <- function(x) {
   if (length(x) < 2) return(NA)
   y <- sort(x, decreasing = TRUE)
-  y[[1]] - y[[2]]
+  abs.diff <- y[[1]] - y[[2]]
+  rel.diff  <- (y[[1]] - y[[2]]) / y[[2]]
+
+  list(rel = rel.diff, abs = abs.diff)
 }
 
 
@@ -393,7 +441,21 @@ strRemovIfString <- function(x, pattern) {
 }
 
 
-getParTableLabels <- function(parTable, labelCol="label") {
+getParTableLabels <- function(parTable, labelCol="label", replace.dup = FALSE) {
+  if (replace.dup) {
+		labels <- unique(parTable[[labelCol]][parTable[[labelCol]] != ""])
+		
+		for (label in labels) {
+			match <- parTable[[labelCol]] == label
+			first <- which.max(match)
+			parTable[match, labelCol] <- ""
+			parTable[first, labelCol] <- label
+		}
+  }
+
+  custom <- parTable$op == ":="
+  parTable[custom, c("op", "rhs")] <- ""
+
   ifelse(parTable[[labelCol]] == "", 
          yes = paste0(parTable$lhs, parTable$op, parTable$rhs),
          no = parTable[[labelCol]])
@@ -410,12 +472,12 @@ padCharMatrix <- function(X, n=1) {
 }
 
 
-timeExpr <- function(x) {
+timeExpr <- function(expr) {
   start <- Sys.time()
-  out <-x
+  out <- expr
   end <- Sys.time()
   print(end - start)
-  out
+  invisible(out)
 }
 
 
@@ -429,3 +491,191 @@ formatParameters <- function(...) {
   paste0(sprintf("%s%s = %s\n", pad, names, values), 
          collapse = "")
 }
+
+
+padString <- function(x, pad) {
+  s <- stringr::str_replace_all(x, pattern = "\n", replacement = paste0("\n", pad))
+  paste0(pad, s, "\n")
+}
+
+
+getCoefsRows <- function(y, x, parTable) {
+  row <- parTable[parTable$lhs %in% y & 
+                  parTable$rhs %in% x & 
+                  parTable$op == "~", , drop = FALSE]
+}
+
+
+getCoefs <- function(y, x, parTable, col = "est") {
+  coefRows <- getCoefsRows(y, x, parTable)
+  coefs <- structure(coefRows[[col]], names = coefRows$rhs)
+  coefs[x]
+}
+
+
+getCOEFS <- function(y, x, COEFS, parTable = NULL) {
+  if (is.null(parTable)) {
+    y <- stringr::str_replace_all(y, ":", OP_REPLACEMENTS[[":"]])
+    x <- stringr::str_replace_all(x, ":", OP_REPLACEMENTS[[":"]])
+    labels <- paste0(y, OP_REPLACEMENTS[["~"]], x)
+
+  } else {
+    rows   <- getCoefsRows(y, x, parTable)
+    names  <- paste0(rows$lhs, rows$op, rows$rhs)
+    labels <- structure(rows$label, names = names)
+    labels <- labels[paste0(y, "~", x)]
+  }
+
+  if (length(labels) == 1) COEFS[[labels]] else COEFS[labels]
+}
+
+
+parTableToEnv <- function(parTable, prefix = "") {
+  lhs <- parTable$lhs
+  rhs <- parTable$rhs
+  op <- stringr::str_replace_all(parTable$op, OP_REPLACEMENTS)
+
+  names <- paste0(prefix, lhs, op, rhs)
+  values <- structure(parTable$est, names = names)
+
+  list2env(values)
+}
+
+
+getConstrExprs <- function(parTable, parTableCov) {
+  parTable <- unique(rbind(parTable, parTableCov))
+  rows <- sortConstrExprs(parTable)
+  if (is.null(rows)) return(NULL)
+
+  fixedParams <- unique(rows$lhs)
+  thetaFixedParams <- vector("list", length(fixedParams))
+  names(thetaFixedParams) <- fixedParams
+
+  exprs <- lapply(rows$rhs, function(expr) parse(text = expr))
+  list(fixedParams = fixedParams, thetaFixedParams = thetaFixedParams,
+       exprs = exprs)
+}
+
+
+sortConstrExprs <- function(parTable) {
+  rows <- parTable[parTable$op %in% CONSTRAINT_OPS, ]
+  if (NROW(rows) == 0) return(NULL)
+
+  labelled <- unique(parTable$mod[parTable$mod != ""])
+  isConst  <- canBeNumeric(rows$rhs)
+  
+  rows <- rows[!(isConst & rows$op %in% BOUNDUARY_OPS), ] # not relevant
+
+  if (!all(rows$lhs %in% labelled)) {
+    stop2("Unknown labels in constraints: ", rows$lhs[!rows$lhs %in% labelled])
+
+  } else if (length(unique(rows$lhs)) != length(rows$lhs)) {
+    stop2("Duplicated labels in constraints:\n", capturePrint(table(rows$lhs)))
+
+  } else if (any(rows$op %in% BOUNDUARY_OPS)) {
+    stop2("Dynamic constraints with ('<', '>') are not implemented yet!")
+  }
+
+  definedLabels <- labelled[!labelled %in% rows$lhs]
+  subRows <- rows
+  sortedRows <- data.frame(lhs = NULL, op = NULL, lhs = NULL, mod = NULL)
+  while (NROW(subRows) > 0) {
+    matchedAny <- FALSE
+    for (i in seq_len(nrow(subRows))) {
+      labels_i <- getVarsExpr(subRows[i, "rhs"])
+      if (length(labels_i) == 0 || all(labels_i %in% definedLabels)) {
+        matchedAny <- TRUE
+        sortedRows <- rbind(sortedRows, subRows[i, ])
+        definedLabels <- c(definedLabels, subRows[i, "lhs"])
+        subRows <- subRows[-i, ]
+        break
+      }
+    }
+
+    stopif(!matchedAny, "Unkown labels in constraints: ",
+           labels_i[!labels_i %in% definedLabels])
+  }
+
+  if (NROW(sortedRows) != NROW(rows)) {
+    warning2("Something went wrong when sorting and parsing constraint-expressions ",
+             "attempting to estimate model with unsorted expressions")
+    return(rows)
+  }
+
+  sortedRows
+}
+
+
+sortConstrExprsFinalPt <- function(parTable) {
+  if (!NROW(parTable)) return(NULL)
+
+  # wrapr for sortConstrExprs meant to be used on the final output parTable
+  # not for the input parTable (e.g., mod -> label)
+  constrExprs <- sortConstrExprs(rename(parTable, label="mod"))
+ 
+  if (is.null(constrExprs)) NULL else rename(constrExprs, mod="label")
+}
+
+
+subsetByGrouping <- function(df, grouping = NULL) {
+  if (is.null(grouping)) return(df)
+  grouping <- grouping[intersect(names(grouping), colnames(df))]
+  df[apply(df[names(grouping)], MARGIN = 1, FUN = \(x) all(x == grouping)), ]
+}
+
+
+GINV <- function(X) {
+  structure(MASS::ginv(X), dimnames = dimnames(X))
+}
+
+
+cov2cor <- function(vcov) {
+  sd <- sqrt(abs(diag(vcov))) # use `abs()`, in case some variances are negative
+  
+  D <- diag(1 / sd)
+  structure(D %*% vcov %*% D, dimnames = dimnames(vcov))
+}
+
+
+leftJoin <- function(left, right, by = intersect(colnames(left), colnames(right))) {
+  ol <- "__orig_order_right__"
+  or <- "__orig_order_left__"
+
+  left[[ol]] <- seq_len(nrow(left))
+  right[[or]] <- seq_len(nrow(right))
+
+  # left join
+  joined <- merge(left, right, by = by, all.x = TRUE)
+
+  # order
+  ordered <- joined[order(joined[[ol]], joined[[or]]), ]
+
+  # return
+  ordered[!colnames(ordered) %in% c(ol, or)]
+}
+
+
+eraseConsoleLines <- function(n = 1) {
+  if (n < 1 || !interactive()) return(invisible())
+  seq <- paste0(rep("\033[1A\033[2K", n), collapse = "")
+  # bring cursor to start of the line we landed on
+  cat(seq, "\033[1G", sep = "")
+}
+
+
+tryFormatC <- function(..., .onfail = "NA") {
+  tryCatch(formatC(...), error = \(e) .onfail)
+}
+
+
+is.diag <- function(M, zero.tol = .Machine$double.eps) {
+  diag(NROW(M)) > zero.tol
+}
+
+
+is.invertible <- function(M) {
+  tryCatch({
+    solve(M)
+    TRUE
+  }, error = \(e) FALSE)
+}
