@@ -21,3 +21,19 @@ modsemEst <- lavaan::parameterEstimates(fit_modsem$lavaan)
 modsemEst[is.na(modsemEst)] <- -999
 testthat::expect_equal(lavaanEst, modsemEst)
 
+set.seed(123)
+oneIntMG <- oneInt
+oneIntMG$group <- sample(c("G1", "G2"), nrow(oneInt), replace = TRUE)
+
+m1 <- '
+  X =~ x1 + x2 + x3
+  Z =~ z1 + z2 + z3
+  Y =~ y1 + y2 + y3
+
+  Y ~ X + Z + X:Z
+'
+
+est <- modsem(m1, oneIntMG, group = "group")
+standardized_estimates(est, correction = TRUE)
+standardized_estimates(est, correction = TRUE, std.errors = "delta")
+summary(est)
